clear
close all
max_bits=30; %signals may be displayed if the transmitted bits are less than max_bits

%input section

samples=input('enter the number of samples per symbol: ');
bits=input('enter the number of information bits: ');
repetitions=input('enter the number of repetitions per bit (must be odd): ');
if rem(repetitions,2)==0 
    disp('error'); 
    return; 
end
cbits=bits*repetitions; %coded bits
pulse_type=input('enter the pulse type (0: NRZ; 1:Raised Cosine); ');
switch pulse_type
    case 0
        pulse=nrz(samples);
        intervals=0;
    case 1
        alfa=input('enter the Roll Off factor (>0, <=1): ');
        if alfa<=0 || alfa >1
            disp('error');
            return
        end
        intervals=input('enter the number of T intervals for representing the pulse: ');
        pulse=square_root_raised_cosine(alfa,samples,intervals);
    otherwise
        disp('error');
        return;
end

%balanced information in antipodal and binary formats
p0=.5;
infobits=rand(1,bits)>p0; %binary 0, 1
info=2*infobits-1;        %0->-1, 1->1

%repetition coding
cinfo=ones(repetitions,1)*info;
cinfo=reshape(cinfo,1,prod(size(cinfo)));
tbits=cinfo>0;

%build the signal
x1=signal_build(cinfo,samples);
x2=conv(x1,pulse);
ts=pulse_type*samples/2; %to adjust the delay to be added for raised cosine
xt=x2(1+intervals*samples-ts:length(x1)+intervals*samples-ts); 

%build the reference signal
callones=ones(1,length(cinfo));
x1allones=signal_build(callones,samples);
x2allones=conv(x1allones,pulse);
xtallones=x2allones(1+intervals*samples-ts:length(x1)+intervals*samples-ts); 

%plot the signal

t0=(1:length(xt))/samples;
    
if cbits < max_bits
    show_plots=input('plot the signals (0/1)?: ');
else
    show_plots=0;
end
if show_plots
    hold off
    plot(t0,xt,'k');
    title('transmitted signal');
    hold on
end

%enter the noise variance and plot the noisy signal

snrdB=input('enter snr (dB): ');
snr=10^(snrdB/10);
noisev=1/(2*snr);
noise=randn(1,length(t0))*noisev^.5;
rn=xt+noise;
if show_plots
    plot(t0,rn,'r');
    title('received signal')
end

%compare the noisy signal with the reference ones

if cbits<max_bits
    disp('transmitted bits')
    str=[];
    for i=1:length(cinfo)
        str=strcat(str,sprintf('%1d',tbits(i)));
    end
    disp(str);
end

%coherent demodulation
rcorr1=rn.*xtallones;
rcorr1=reshape(rcorr1,samples,cbits);
received=sum(rcorr1,1);
r2bits=received>0;
if cbits<max_bits
    disp('Hard decoding correlator output');
    str=[];
    for i=1:length(cinfo)
        str=strcat(str,sprintf('%1d',r2bits(i)));
    end
    disp(str);
end

%matched filter
mfagain=1;
synchroupdate=0;
while mfagain==1
    x0=samples*(1:cbits)+(intervals*samples-ts)+pulse_type;
    mfoutput=conv(rn,pulse);
    mfsamples=mfoutput(x0+synchroupdate);
    mfbits=mfsamples>0;

%error count
    errorscorr=sum(xor(tbits,r2bits));
    errorsmf=sum(xor(tbits,mfbits));
    str=sprintf(' errors correlator = %d, errors matched filter = %d',errorscorr,errorsmf);
    disp(str);

    another=input('do you want to adjust the synchronization (0/1)?: ');
    if another
        synchroupdate=input('enter a integer value: ');
    else
        mfagain=0;
    end
end
ber=1/2*erfc(snr^.5);
str=sprintf(' B.E.R correlator:%.3e, B.E.R. matched filter: %.3e, theoretical B.E.R.:%.3e',errorscorr/cbits,errorsmf/cbits,ber);
disp(str);

%decoding
if repetitions>1
    rhard=reshape(r2bits,repetitions,bits);
    rsoft=reshape(received,repetitions,bits);
    errorsh=sum(xor(infobits,sum(rhard)>repetitions/2));
    errorshp=find(xor(infobits,sum(rhard)>repetitions/2)==1);
    errorss=sum(xor(infobits,sum(rsoft)>0));
    errorssp=find(xor(infobits,sum(rsoft)>0)==1);
    str=sprintf('errors h = %d, errors s = %d',errorsh,errorss);
    disp(str);
    ferh=ber^repetitions;
    for i=1:floor(repetitions/2)
        ferh=ferh+bin(repetitions,i)*ber^(repetitions-i)*(1-ber)^i;
    end
    fers=1/2*erfc((repetitions*snr)^.5);
    str=sprintf('F.E.R. hard:%.3e, soft:%.3e',errorsh/bits,errorss/bits);
    disp(str);
    str=sprintf('theoretical F.E.R. hard:%.3e, soft:%.3e',ferh,fers);
    disp(str);
    if errorshp
        showerrors=input('show the decoding errors? (0/1) ');
        if showerrors
            %display the received samples and the relevant errors
            both = ismember(errorshp', errorssp', 'rows');
            both2 = ismember(errorssp', errorshp', 'rows');
            for i=1:length(errorshp)
                if both(i)
                    cprintf('Errors','\n hard and soft error\n');
                else
                    cprintf('Errors','\n hard error only\n');
                end
                pos=errorshp(i);
                str=sprintf('transmitted bit: %d',infobits(pos));
                disp(str);
                str=('hard values: ');
                for j=1:repetitions-1
                    str1=sprintf('%d,',rhard(j,pos));
                    str=strcat(str,str1);
                end
                str1=sprintf('%d',rhard(repetitions,pos));
                str=strcat(str,str1);
                disp(str);
                str=('soft values: ');
                for j=1:repetitions
                    str1=sprintf('%.3f,',rsoft(j,pos));
                    str=strcat(str,str1);
                end
                str1=sprintf('sum=%.3f',sum(rsoft(:,pos)));
                str=strcat(str,str1);
                disp(str);
                dummy=input('push a key');
            end
            for i=1:length(errorssp)
                if both2(i)==0
                    cprintf('Errors','\n soft error only\n');
                    pos=errorssp(i);
                    str=sprintf('transmitted bit: %d',infobits(pos));
                    disp(str);
                    str=('hard values: ');
                    for j=1:repetitions-1
                        str1=sprintf('%d,',rhard(j,pos));
                        str=strcat(str,str1);
                    end
                    str1=sprintf('%d',rhard(repetitions,pos));
                    str=strcat(str,str1);
                    disp(str);
                    str=('soft values: ');
                    for j=1:repetitions
                        str1=sprintf('%.3f,',rsoft(j,pos));
                        str=strcat(str,str1);
                    end
                    str1=sprintf('sum=%.3f',sum(rsoft(:,pos)));
                    str=strcat(str,str1);
                    disp(str);
                    dummy=input('push a key');
                end
            end
        end
    end
end
if show_plots 
    dummy=input('push a key');
    
    figure
    stem(received,'r*');
    title('comparison among transmitted values and correlator outputs');
    hold on
    stem(cinfo,'ko');
    
    figure
    plot(t0,mfoutput(1+intervals*samples-ts:length(x1)+intervals*samples-ts))
    title('matchetd filter output');
    hold on, stem(1:cbits,cinfo,'r');

    figure
    x=-2:2;
    y=zeros(1,length(x));
    plot(x,y,'k-');
    hold on
    title('constellation');
    x=[0,0];
    y=[-1,1];
    plot(x,y,'k-');
    x=[-1,1]; y=[0,0];
    plot(x,y,'x');
    for i=1:length(cinfo), plot(received(i),0,'r*'); end
    
    ploteye(mfoutput,samples,x0(1)-samples/2,cbits);
end
