function []=scattering(signals,snr_in_dB,N)
% []=scattering(signals,snr_in_dB,N)
%		SCATTERING  plots the scattering diagram for a given
%   		value of snr_in_dB, signal to noise ratio in dB (Eb/N0).
%               signals: each row represents the square root energies of the signals (x,y);
%               N = number of simulated symbols
 
  color=['y','k','b','c','m','y','g','w'];

  plsize=max(max(signals))*1.5;

  sigsize=size(signals,1);
  basedim=size(signals,2);
  if ~(basedim==2)
     disp('error, bidimensional only');
     return;
  end
  
  avenergy=sum(sum(signals.^2))/sigsize
  SNR=10^(snr_in_dB/10)*log2(sigsize);	% signal to noise ratio
  sgma=(avenergy)^0.5/sqrt(2*SNR)   	% sigma, standard deviation of noise
                                        % signal energy equal to one
% data generation
  dsource=floor(sigsize*rand(1,N))+1;
% the detection, and probability of error calculation follows
% The matched filter outputs
  x=signals(dsource,1)'+sgma*randn(1,N);
  y=signals(dsource,2)'+sgma*randn(1,N);
% plot scattering
  figure;
  hold on;
  axis('square');
  axis([-plsize plsize -plsize plsize]);
  if (1)
     for i=-plsize:.03*plsize:plsize
        for j=-plsize:.03*plsize:plsize
           for h=1:sigsize, sqdist(h,:)=(signals(h,1)-i).^2+(signals(h,2)-j).^2;end
           [a, decis]=min(sqdist);
           plot(i,j,strcat('.',color(rem(ceil((atan2(signals(decis,1),signals(decis,2))+pi)*8/(2*pi)+3*norm(signals(decis,:))),8)+1)));
        end
     end
  end
  plot(x,y,'.k');
  for i=1:sigsize, plot(signals(i,1),signals(i,2),'+r'); end
  hold off